#
# Copyright (C) 2012 Alexis Bienvenue <paamc@passoire.fr>
#
# This file is part of Auto-Multiple-Choice
#
# Auto-Multiple-Choice is free software: you can redistribute it
# and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, either version 2 of
# the License, or (at your option) any later version.
#
# Auto-Multiple-Choice is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Auto-Multiple-Choice.  If not, see
# <http://www.gnu.org/licenses/>.

package AMC::Export::rawsqlite;

use AMC::Basic;
use AMC::Export;

use AMC::DataModule::capture ':zone';

use Encode;

@ISA=("AMC::Export");

sub new {
    my $class = shift;
    my $self  = $class->SUPER::new();
    bless ($self, $class);
    return $self;
}

sub load {
  my ($self)=@_;
  $self->SUPER::load();
  $self->{'_capture'}=$self->{'_data'}->module('capture');
}

sub export {
    my ($self,$filename)=@_;

    $self->load();

    my $data=$self->{'_data'};

    unlink($filename) if(-f $filename);

    $data->{'dbh'}->{AutoCommit}=1;
    $data->sql_do("ATTACH DATABASE "
		  .$data->sql_quote($filename)
		  ." AS rawsqlite");
    $data->{'dbh'}->{AutoCommit}=0;

    $data->begin_transaction('RWsq');

    my $dt=$self->{'_scoring'}->variable('darkness_threshold');

    $data->sql_do("CREATE TABLE rawsqlite.ticked"
		  ." (id TXT, student INTEGER, copy INTEGER,"
		  ."  question INTEGER, answer INTEGER, ticked INTEGER,"
		  ."  q_answered INTEGER, q_invalid INTEGER,"
		  ."   PRIMARY KEY (student,copy,question,answer))");

    $data->sql_do("INSERT INTO rawsqlite.ticked (student,copy,question,answer,ticked)"
		  ." SELECT student,copy,id_a AS question, id_b AS answer,"
		  ." CASE"
		  ." WHEN manual >= 0 THEN manual"
		  ." WHEN total<=0 THEN -1"
		  ." WHEN black >= ? * total THEN 1"
		  ." ELSE 0"
		  ." END AS ticked"
		  ." FROM capture.capture_zone"
		  ." WHERE type=?",
		  $dt,ZONE_BOX);

    $data->sql_do("UPDATE rawsqlite.ticked"
		  ." SET q_answered=(SELECT"
		  ."    CASE WHEN why=? THEN 0 ELSE 1 END"
		  ."    FROM scoring.scoring_score"
		  ."    WHERE scoring_score.student=ticked.student"
		  ."    AND scoring_score.copy=ticked.copy"
		  ."    AND scoring_score.question=ticked.question)",
		  "V");
    $data->sql_do("UPDATE rawsqlite.ticked"
		  ." SET q_invalid=(SELECT"
		  ."    CASE WHEN why=? THEN 1 ELSE 0 END"
		  ."    FROM scoring.scoring_score"
		  ."    WHERE scoring_score.student=ticked.student"
		  ."    AND scoring_score.copy=ticked.copy"
		  ."    AND scoring_score.question=ticked.question)",
		  "E");

    $data->sql_do("UPDATE rawsqlite.ticked"
		  ." SET id=(SELECT CASE"
		  ."         WHEN manual IS NOT NULL THEN manual"
		  ."         ELSE auto END"
		  ."         FROM association.association_association"
		  ."   WHERE association_association.student=ticked.student"
		  ."   AND association_association.copy=ticked.copy)");

    $data->end_transaction('RWsq');
}

1;
